# DoubleVIP release 3 by David Bowland
# ./addons/eventscripts/doublevip/doublevip.py

# >>> To configure this addon please see doublevip.cfg <<<

"""
Picks one player on each team the team must protect to survive. Requires EZRestrict and ES 2.0+
"""


import es
from ezrestrict import ezrestrict
import langlib
import os.path
import playerlib
import random


info = es.AddonInfo()
info.name = 'DoubleVIP'
info.version = '3'
info.url = 'http://addons.eventscripts.com/addons/view/doublevip'
info.basename = 'doublevip'
info.author = 'SuperDave'


bool_added_prefix = 0
dict_vips = {}

dict_vip_colors = {0:{'string':'255 255 255', 'int':16777215}, 1:{'string':'255 0 0', 'int':255}, 2:{'string':'0 0 255', 'int':16711680}, 3:{'string':'0 255 0', 'int':65280}, 4:{'string':'255 255 0', 'int':65535}, 5:{'string':'100 50 0', 'int':12900}, 6:{'string':'60 60 60', 'int':3947580}, 7:{'string':'0 0 0', 'int':0}}
dict_options = {'doublevip_vip_health':[250, 'Starting health of VIPs'], 'doublevip_bot_avoid':[1, '0 = all players have an equal chance of becoming VIP, 1 = human players will be preferred for VIP, 2 = human players will be picked for VIP when possible'], 'doublevip_bot_speed':[4, 'Speed of VIP bots from 0 (half speed) to 5 (full speed)'], 'doublevip_vip_restrict_primary':[1, '0 = VIPs are allowed primary weapons, 1 = VIPs will be restricted from primary weapons'], 'doublevip_vip_restrict_secondary':[0, '0 = VIPs are allowed secondary weapons, 1 = VIPs will be restricted from secondary weapons'], 'doublevip_vip_restrict_grenades':[0, '0 = VIPs are allowed grenades, 1 = VIPs will be restricted from grenades'], 'doublevip_vip_restrict_remove':[0, '0 = VIPs will drop restricted weapons, 1 = restrcited weapons picked up by a VIP will be removed'], 'doublevip_vip_color_t':[1, 'Color of Terrorist VIP: 0 = no change, 1 = red, 2 = blue, 3 = green, 4 = yellow, 5 = brown, 6 = gray, 7 = black'], 'doublevip_vip_color_ct':[2, 'Color of Counter-Terrorist VIP: 0 = no change, 1 = red, 2 = blue, 3 = green, 4 = yellow, 5 = brown, 6 = gray, 7 = black'], 'doublevip_vip_transparency':[75, 'Transparency of VIPs from 0 (invisible) to 255 (opaque/no change)'], 'doublevip_removeobjectives':[1, '0 = no change, 1 = map objectives will be removed'], 'doublevip_announce_hud':[1, '0 = no change, 1 = announce addon objectives to players at round start with a hud hint'], 'doublevip_announce_chat':[0,'0 = no change, 1 = announce to players when they are chosen to be a VIP in chat area, 2 = announce to team members when a player is chosen to be a VIP in chat area, 3 = announce to all players when a player is chosen to be a VIP in chat area'], 'doublevip_announce_center':[1, '0 = no change, 1 = announce to players when they are chosen to be a VIP in center text, 2 = announce to team members when a player is chosen to be a VIP in center text'], 'doublevip_announce_console':[2, '0 = no change, 1 = announce to player when they are chosen to be a VIP on the console, 2 = announce to team members when a player is chosen to be a VIP on the console, 3 = announce to all players when a player is chosen to be a VIP on the console']}
func_lang_text = None


def load():
   """
   Checks to make sure critical server variables are created by the config
   Adds the 'd' and 'D' characters to eventscripts_cmdprefixes to catch the "drop" command
   """
   global bool_added_prefix
   global func_lang_text

   for str_option in dict_options:
      es.ServerVar(str_option, dict_options[str_option][0], dict_options[str_option][1])
   if os.path.isfile(es.getAddonPath('doublevip') + '/doublevip.cfg'):
      es.server.cmd('es_xmexec ../addons/eventscripts/doublevip/doublevip.cfg')
   else:
      es.dbgmsg(0, 'DoubleVIP: Unable to load doublevip.cfg! Please ensure it is in the ./doublevip/ directory.')

   str_lang_path = es.getAddonPath('doublevip') + '/doublevip_languages.ini'
   if os.path.isfile(str_lang_path):
      func_lang_text = langlib.Strings(str_lang_path)
   else:
      es.dbgmsg(0, 'DoubleVIP: Unable to load doublevip_languages.ini! Please ensure it is in the ./doublevip/ directory.')
   if not func_lang_text:
      func_lang_text = missing_lang

   if 'd' not in str(es.ServerVar('eventscripts_cmdprefixes')) or 'D' not in str(es.ServerVar('eventscripts_cmdprefixes')):
      bool_added_prefix = 1
      es.ServerVar('eventscripts_cmdprefixes').set(es.ServerVar('eventscripts_cmdprefixes') + 'dD')


def es_map_start(event_var):
   """Clears the dictionary of VIPs"""
   global dict_vips

   dict_vips.clear()


def es_client_command(event_var):
   """Calls color_weapons if the player uses the "drop" command"""
   if event_var['command'] == 'drop':
      int_userid = int(event_var['userid'])
      if dict_vips.has_key(int_userid):
         color_weapons(int_userid)


def round_start(event_var):
   """
   Calls reset_vips
   Loads variables common to both VIPs
   Generates a selection of players and chooses a VIP
   Announces the VIP selections
   """
   global dict_vips

   list_last_vips = list(dict_vips)
   reset_vips()

   dict_player_list = es.createplayerlist()
   list_vip_restrictions = []
   if int(es.ServerVar('doublevip_vip_restrict_primary')):
      list_vip_restrictions.append('#primary')
   if int(es.ServerVar('doublevip_vip_restrict_secondary')):
      list_vip_restrictions.append('#secondary')
   if int(es.ServerVar('doublevip_vip_restrict_grenades')):
      list_vip_restrictions.append('#grenade')
   int_bot_speed = int(es.ServerVar('doublevip_bot_speed'))
   int_vip_alpha = int(es.ServerVar('doublevip_vip_transparency'))
   int_announce_chat = int(es.ServerVar('doublevip_announce_chat'))
   int_announce_center = int(es.ServerVar('doublevip_announce_center'))
   int_announce_console = int(es.ServerVar('doublevip_announce_console'))

   for int_team in (2, 3):
      if es.getplayercount(int_team):
         list_possible_vips = []
         list_possible_human_vips = []
         for int_userid in dict_player_list:
            if int_team == dict_player_list[int_userid]['teamid']:
               list_possible_vips.append(int_userid)
               if dict_player_list[int_userid]['steamid'] <> 'BOT':
                  list_possible_human_vips.append(int_userid)
         int_avoid_bots = int(es.ServerVar('doublevip_bot_avoid'))
         for int_last_vip in list_last_vips:
            if int_last_vip in list_possible_vips and len(list_possible_vips) > 1:
               list_possible_vips.remove(int_last_vip)
            if int_last_vip in list_possible_human_vips and int_avoid_bots <> 2:
               list_possible_human_vips.remove(int_last_vip)
         if int_avoid_bots and list_possible_human_vips:
            int_vip_userid = random.choice(list_possible_human_vips)
         else:
            int_vip_userid = random.choice(list_possible_vips)

         es.setplayerprop(int_vip_userid, 'CBasePlayer.m_iHealth', es.ServerVar('doublevip_vip_health'))
         int_vip_color = int(es.ServerVar('doublevip_vip_color_' + {2:'t', 3:'ct'}[int_team]))
         es.server.cmd('es_xfire %s !self addoutput \"rendermode 1\"' % int_vip_userid)
         if int_vip_color:
            es.server.cmd('es_xfire %s !self color \"%s\"' % (int_vip_userid, dict_vip_colors[int_vip_color]['string']))
         if int_vip_alpha <> 255:
            es.server.cmd('es_xfire %s !self alpha %s' % (int_vip_userid, int_vip_alpha))
         if es.getplayersteamid(int_vip_userid) == 'BOT' and int_bot_speed <> 5:
            es.setplayerprop(int_vip_userid, 'CBasePlayer.localdata.m_flLaggedMovementValue', float(int_bot_speed + 5) / 10)
         if list_vip_restrictions:
            ezrestrict.restrict(int_vip_userid, list_vip_restrictions)

         dict_vips[int_vip_userid] = {'restrictions':list_vip_restrictions, 'color':int_vip_color, 'alpha':int_vip_alpha, 'team':int_team, 'weapons':[]}
         color_weapons(int_vip_userid)

         str_team_text = {2:'vip t', 3:'vip ct'}[int_team]
         for int_userid in dict_player_list:
            str_player_lang = playerlib.getPlayer(int_userid).get('lang')
            if int_announce_chat >= 1 and int_userid == int_vip_userid:
               es.tell(int_userid, '#multi', func_lang_text('vip player', {}, str_player_lang))
            elif int_announce_chat >= 2 and int_team <> 5 - dict_player_list[int_userid]['teamid']:
               es.tell(int_userid, '#multi', func_lang_text(str_team_text, {'vip':es.getplayername(int_vip_userid)}, str_player_lang))
            elif int_announce_chat >= 3:
               es.tell(int_userid, '#multi', func_lang_text(str_team_text, {'vip':es.getplayername(int_vip_userid)}, str_player_lang))
            if int_announce_center >= 1 and int_userid == int_vip_userid:
               es.centertell(int_userid, remove_tags(func_lang_text('vip player', {}, str_player_lang)))
            elif int_announce_center >= 2 and int_team <> 5 - dict_player_list[int_userid]['teamid']:
               es.centertell(int_userid, remove_tags(func_lang_text(str_team_text, {'vip':es.getplayername(int_vip_userid)}, str_player_lang)))
            if int_announce_console >= 1 and int_userid == int_vip_userid:
               es.cexec(int_userid, 'echo ' + remove_tags(func_lang_text('vip player', {}, str_player_lang)))
            elif int_announce_console >= 2 and int_team <> 5 - dict_player_list[int_userid]['teamid']:
               es.cexec(int_userid, 'echo ' + remove_tags(func_lang_text(str_team_text, {'vip':es.getplayername(int_vip_userid)}, str_player_lang)))
            elif int_announce_console >= 3:
               es.cexec(int_userid, 'echo ' + remove_tags(func_lang_text(str_team_text, {'vip':es.getplayername(int_vip_userid)}, str_player_lang)))

   int_userid = es.getuserid()
   if int_userid and int(es.ServerVar('doublevip_removeobjectives')):
      es.server.cmd('es_xfire %s weapon_c4 kill' % int_userid)
      es.server.cmd('es_xfire %s func_bomb_target kill' % int_userid)
      es.server.cmd('es_xfire %s hostage_entity kill' % int_userid)
      es.server.cmd('es_xfire %s func_hostage_rescue kill' % int_userid)
   int_hud = int(es.ServerVar('doublevip_announce_hud'))
   if int_userid and int_hud:
      es.server.cmd('es_xfire %s env_hudhint kill' % int_userid)
      es.server.cmd('es_xgive %s env_hudhint' % int_userid)
   for int_userid in dict_player_list:
      if int(dict_player_list[int_userid]['teamid']) > 1:
         if int_hud:
            if dict_vips.has_key(int_userid):
               es.server.cmd('es_xfire %s env_hudhint addoutput \"message %s\"' % (int_userid, remove_tags(func_lang_text('vip player', {}, playerlib.getPlayer(int_userid).get('lang')))))
            else:
               es.server.cmd('es_xfire %s env_hudhint addoutput \"message %s\"' % (int_userid, remove_tags(func_lang_text('objective vip', {}, playerlib.getPlayer(int_userid).get('lang')))))
         es.server.cmd('es_xfire %s env_hudhint showhudhint' % int_userid)


def item_pickup(event_var):
   """Calls color_weapons when a VIP picks up an item"""
   int_userid = int(event_var['userid'])
   if dict_vips.has_key(int_userid):
      color_weapons(int_userid)


def player_death(event_var):
   """Kills the player's team if the player is a VIP"""
   int_userid = int(event_var['userid'])
   if dict_vips.has_key(int_userid):
      int_attacker = int(event_var['attacker'])
      int_team = dict_vips[int_userid]['team']
      if int_attacker:
         kill_team(int_team, int_attacker)
      else:
         kill_team(int_team, int_userid)


def player_disconnect(event_var):
   """
   Kills the player's team if the player is a VIP
   Resets the color the player's weapons
   """
   global dict_vips

   int_userid = int(event_var['userid'])
   if dict_vips.has_key(int_userid):
      int_attacker = 0
      int_team = dict_vips[int_userid]['team']
      for int_loop_userid in dict_vips:
         if int_loop_userid <> int_userid:
            int_attacker = int_loop_userid
            break
      if int_attacker:
         kill_team(int_team, int_attacker)

      dict_entity_list = es.createentitylist()
      for int_weapon_index in dict_vips[int_userid]['weapons']:
         if dict_entity_list.has_key(int_weapon_index):
            es.setindexprop(int_weapon_index, 'CBaseEntity.m_nRenderMode', es.getindexprop(int_weapon_index, 'CBaseEntity.m_nRenderMode') | 1)
            es.setindexprop(int_weapon_index, 'CBaseEntity.m_nRenderFX', es.getindexprop(int_weapon_index, 'CBaseEntity.m_nRenderFX') | 256)
            es.setindexprop(int_weapon_index, 'CBaseEntity.m_clrRender', 4294967295)
      del dict_vips[int_userid]


def round_end(event_var):
   """
   Determines the winning team
   Finds a player on the winning team
   Removes a point from the winning team if that team did not kill the VIP
   """
   int_team = 0
   if event_var['reason'] == '13':
      int_team = 2
   if event_var['reason'] == '12':
      int_team = 3

   if int_team and int(es.ServerVar('doublevip_removeobjectives')):

      int_score_userid = 0
      dict_player_list = es.createplayerlist()
      for int_userid in dict_player_list:
        if int_team == dict_player_list[int_userid]['teamid']:
         int_score_userid = int_userid
         break

      if int_score_userid:
         dict_game_scores = es.createentitylist('game_score')
         int_game_score_index = es.getentityindex('game_score')
         if len(dict_game_scores) > 1:
            for int_index in dict_game_scores:
               if int_index <> int_game_score_index:
                  es.server.cmd('es_xremove %s' % int_index)
         elif int_game_score_index == -1:
            es.server.cmd('es_xgive %s game_score' % int_score_userid)
         es.server.cmd('es_xfire %s game_score addoutput \"spawnflags 3\"' % int_score_userid)
         es.server.cmd('es_xfire %s game_score addoutput \"points -1\"' % int_score_userid)
         es.server.cmd('es_xfire %s game_score applyscore' % int_score_userid)


def ezrestricted_pickup(event_var):
   """Removes any restricted weapons the VIP tries to pick up"""
   if dict_vips.has_key(int(event_var['userid'])) and int(es.ServerVar('doublevip_vip_restrict_remove')):
      ezrestrict.saferemove(int(event_var['weaponindex']))


def unload():
   """Calls reset_vips"""
   reset_vips()

   if bool_added_prefix:
      str_temp = str(es.ServerVar('eventscripts_cmdprefixes'))
      es.ServerVar('eventscripts_cmdprefixes').set(str_temp.replace('dD', ''))

   for str_option in dict_options:
      es.ServerVar(str_option).set(0)


def color_weapons(int_userid):
   """Colors the weapons of int_userid to be consistent with the player's color"""
   global dict_vips

   dict_current_vip = dict_vips[int_userid]
   if dict_current_vip['color'] or dict_current_vip['alpha'] <> 255:
      int_vip_handle = es.getplayerhandle(int_userid)
      list_to_remove = []
      dict_entity_list = es.createentitylist()
      float_entity_color = dict_vip_colors[dict_current_vip['color']]['int'] + (dict_current_vip['alpha'] * 16777216)
      for int_weapon_index in dict_current_vip['weapons']:
         if not dict_entity_list.has_key(int_weapon_index):
            list_to_remove.append(int_weapon_index)
         else:
            if int_vip_handle <> es.getindexprop(int_weapon_index, 'CBaseEntity.m_hOwnerEntity'):
               list_to_remove.append(int_weapon_index)
               es.setindexprop(int_weapon_index, 'CBaseEntity.m_nRenderMode', es.getindexprop(int_weapon_index, 'CBaseEntity.m_nRenderMode') | 1)
               es.setindexprop(int_weapon_index, 'CBaseEntity.m_nRenderFX', es.getindexprop(int_weapon_index, 'CBaseEntity.m_nRenderFX') | 256)
               es.setindexprop(int_weapon_index, 'CBaseEntity.m_clrRender', 4294967295)
      for int_weapon_index in list_to_remove:
         dict_current_vip['weapons'].remove(int_weapon_index)
      for int_weapon_index in dict_entity_list:
         if dict_entity_list[int_weapon_index]['classname'][0:7] == 'weapon_' and int_vip_handle == es.getindexprop(int_weapon_index, 'CBaseEntity.m_hOwnerEntity') and int_weapon_index not in dict_current_vip['weapons']:
            dict_current_vip['weapons'].append(int_weapon_index)
            es.setindexprop(int_weapon_index, 'CBaseEntity.m_nRenderMode', es.getindexprop(int_weapon_index, 'CBaseEntity.m_nRenderMode') | 1)
            es.setindexprop(int_weapon_index, 'CBaseEntity.m_nRenderFX', es.getindexprop(int_weapon_index, 'CBaseEntity.m_nRenderFX') | 256)
            es.setindexprop(int_weapon_index, 'CBaseEntity.m_clrRender', float_entity_color)


def reset_vips():
   """Calers the VIP status of the player in dict_vips"""
   global dict_vips

   dict_entity_list = es.createentitylist()
   for int_vip_userid in dict_vips:
      dict_current_vip = dict_vips[int_vip_userid]
      if dict_current_vip['color']:
         es.server.cmd('es_xfire %s !self color \"255 255 255\"' % int_vip_userid)
      if dict_current_vip['alpha'] <> 255:
         es.server.cmd('es_xfire %s !self alpha 255' % int_vip_userid)
      list_vip_restrictions = dict_current_vip['restrictions']
      if list_vip_restrictions:
         ezrestrict.unrestrict(int_vip_userid, list_vip_restrictions)
      for int_weapon_index in dict_current_vip['weapons']:
         if dict_entity_list.has_key(int_weapon_index):
            es.setindexprop(int_weapon_index, 'CBaseEntity.m_nRenderMode', es.getindexprop(int_weapon_index, 'CBaseEntity.m_nRenderMode') | 1)
            es.setindexprop(int_weapon_index, 'CBaseEntity.m_nRenderFX', es.getindexprop(int_weapon_index, 'CBaseEntity.m_nRenderFX') | 256)
            es.setindexprop(int_weapon_index, 'CBaseEntity.m_clrRender', 4294967295)

   dict_vips.clear()


def kill_team(int_team, int_attacker):
   """Kills every member of int_team with int_attacker as the attacker"""
   dict_team_players = es.createplayerlist()
   list_to_remove = []
   for int_userid in dict_team_players:
      if int_team <> dict_team_players[int_userid]['teamid'] or dict_team_players[int_userid]['isdead']:
         list_to_remove.append(int_userid)
   for int_userid in list_to_remove:
      del dict_team_players[int_userid]

   if dict_team_players:
      int_max_health = 0
      for int_userid in dict_team_players:
         es.server.cmd('es_xfire %s !self addoutput \"targetname doublevipslay\"' % int_userid)
         int_health = es.getplayerprop(int_userid, 'CBasePlayer.m_iHealth')
         if int_health > int_max_health:
            int_max_health = int_health

      es.server.cmd('es_xfire %s point_hurt kill' % int_attacker)
      es.server.cmd('es_xgive %s point_hurt' % int_attacker)
      es.server.cmd('es_xfire %s point_hurt addoutput \"damagetype 32\"' % int_attacker)
      es.server.cmd('es_xfire %s point_hurt addoutput \"damagetarget doublevipslay\"' % int_attacker)
      es.server.cmd('es_xfire %s point_hurt addoutput \"damage %s\"' % (int_attacker, int_max_health))
      es.server.cmd('es_xfire %s point_hurt hurt' % int_attacker)

      for int_userid in dict_team_players:
         es.server.cmd('es_xfire %s !self addoutput \"targetname none\"' % int_userid)


def remove_tags(str_text):
   """Removes #lightgreen, #green and #default tags from the supplied string"""
   return str_text.replace('#lightgreen', '').replace('#green', '').replace('#default', '')


def missing_lang(str_heading, dict_tokens, str_language):
   """Displays an error message because the languages .ini could not be found"""
   return 'DoubleVIP: doublevip_languages.ini not found'